@@ -67,7 +67,7 @@
 #' group <- c(rep('D1', N), rep('D2', N))
 #' models <- 'F1 = 1-15'
 #' mod_Rasch <- multipleGroup(dat, models, itemtype = 'Rasch', group = group)
-#' coef(mod_Rasch)
+#' coef(mod_Rasch, simplify=TRUE)
 #' pf <- personfit(mod_Rasch, method='MAP')
 #' head(pf)
 #'
@@ -137,7 +137,7 @@ personfit <- function(x, method = 'EAP', Theta = NULL, stats.only = TRUE, ...){
         for(i in 1L:length(x@Model$itemtype))
             oneslopes[i] <- closeEnough(x@ParObjects$pars[[i]]@par[1L], 1-1e-10, 1+1e-10)
         if(all(oneslopes)){
-            W <- resid <- info <- C <- matrix(0, ncol=J, nrow=N)
+            W <- resid <- C <- matrix(0, ncol=J, nrow=N)
             K <- x@Data$K
             for (i in 1L:J){
                 P <- ProbTrace(x=pars[[i]], Theta=Theta)
